VERSION 4.00
Begin VB.Form Form1
   BackColor       =   &H00FFFFFF&
   Caption         =   "Rotating Cube"
   ClientHeight    =   4140
   ClientLeft      =   1140
   ClientTop       =   1515
   ClientWidth     =   6690
   Height          =   4545
   Left            =   1080
   LinkTopic       =   "Form1"
   ScaleHeight     =   4140
   ScaleWidth      =   6690
   Top             =   1170
   Width           =   6810
End
Attribute VB_Name = "Form1"
Attribute VB_Creatable = False
Attribute VB_Exposed = False

'*****************************************************************************
'                                                                            *
'  CUBE1.FRM                                                                 *
'                                                                            *
'  This program uses Fastgraph's 3D geometry system to draw a cube. Each     *
'  of the six cube faces is a different solid color. Uses the approach of    *
'  projecting each cube face separately.                                     *
'                                                                            *
'*****************************************************************************

Const vbWidth = 640
Const vbHeight = 480

Private Type Point2D
   X As Long
   Y As Long
End Type

Private Type Point3D
   X As Long
   Y As Long
   Z As Long
End Type

Dim hPal As Long
Dim hvb As Long
Dim cxClient As Long, cyClient As Long
Dim AppIsActive As Boolean

Dim bpp As Long
Dim Redo As Boolean
Dim xAngle As Long, yAngle As Long, zAngle As Long

Option Base 1

' 3D transformation matrix
Dim Transform(12) As Long

' six faces of a 40x40x40 cube, defined in 3D coordinates
Dim Face1(4) As Point3D
Dim Face2(4) As Point3D
Dim Face3(4) As Point3D
Dim Face4(4) As Point3D
Dim Face5(4) As Point3D
Dim Face6(4) As Point3D

' indices into the cube array that define each face of the cube
Dim Faces(6) As Long

' work array required by fg_polyfill()
Dim WorkArray(1000) As Point2D

Private Sub Form_Activate()
   Call fg_realize(hPal)
   Refresh
End Sub

Private Sub Form_Load()
   ScaleMode = 3
   Call fg_setdc(hDC)
   hPal = fg_defpal()
   Call fg_realize(hPal)

   Call fg_vbinit
   hvb = fg_vballoc(vbWidth, vbHeight)
   Call fg_vbopen(hvb)
   Call fg_vbcolors

   bpp = fg_colors()
   If bpp > 8 Then Call fg_vbdepth(bpp)
   hvb = fg_vballoc(vbWidth, vbHeight)
   Call fg_vbopen(hvb)
   Call fg_vbcolors
   Call fg_setcolor(-1)
   Call fg_fillpage

   ' define 3D viewport and initial transformation matrix
   Call fg_view3d(0, vbWidth - 1, 0, vbHeight - 1, &H8000&)
   Call fg_rotate3d(Transform(1), 0, 0, 0, 0)
   Call fg_move3d(Transform(1), 0, 0, &HFF9C0000, 0)

   Call InitGlobals

   Show 0
   AppIsActive = True
   While AppIsActive
      Call CheckForMovement
      DoEvents
   Wend
End Sub

Private Sub Form_Paint()
   Call fg_vbscale(0, vbWidth - 1, 0, vbHeight - 1, 0, cxClient - 1, 0, cyClient - 1)
End Sub

Private Sub Form_Resize()
   cxClient = ScaleWidth
   cyClient = ScaleHeight
   Refresh
End Sub

Private Sub Form_Unload(Cancel As Integer)
   AppIsActive = False
   Call fg_vbclose
   Call fg_vbfree(hvb)
   Call fg_vbfin
End Sub

'*****************************************************************************
'                                                                            *
'  CheckForMovement()                                                        *
'                                                                            *
'  The CheckForMovement() function checks if any of the four arrow keys are  *
'  pressed, and if so, pans in that direction if possible.                   *
'                                                                            *
'*****************************************************************************

Private Sub CheckForMovement()
   Dim ShiftKey As Boolean

   ' erase the previous frame from the virtual buffer
   Call fg_setcolor(-1)
   Call fg_fillpage

   ' project the cube onto the virtual buffer and draw it
   Call DrawCube

   ' display what we just drew
   Call fg_vbscale(0, vbWidth - 1, 0, vbHeight - 1, 0, cxClient - 1, 0, cyClient - 1)

   ' check if either shift key is pressed
   ShiftKey = (fg_kbtest(42) = 1) Or (fg_kbtest(54) = 1)

   ' + and - move cube along the z axis (+ is toward viewer, - is
   ' away from viewer), restricting movement beyond z=-40
   If fg_kbtest(74) = 1 Then
      Call fg_move3d(Transform(1), 0, 0, &HFFFD0000, 1)
   ElseIf fg_kbtest(78) = 1 Then
      If Transform(12) < &HFFD80000 Then
         Call fg_move3d(Transform(1), 0, 0, &H30000, 1)
      End If

   ' left and right arrow keys move cube along x axis
   ElseIf fg_kbtest(75) = 1 Then
      Call fg_move3d(Transform(1), &HFFFD0000, 0, 0, 1)
   ElseIf fg_kbtest(77) = 1 Then
      Call fg_move3d(Transform(1), &H30000, 0, 0, 1)

   ' up and down arrow keys move cube along y axis
   ElseIf fg_kbtest(72) = 1 Then
      Call fg_move3d(Transform(1), 0, &H30000, 0, 1)
   ElseIf fg_kbtest(80) = 1 Then
      Call fg_move3d(Transform(1), 0, &HFFFD0000, 0, 1)

   ' x rotates counterclockwise around x axis, X rotates clockwise
   ElseIf fg_kbtest(45) = 1 Then
      If ShiftKey Then
         xAngle = xAngle + 6
         If xAngle >= 360 Then xAngle = xAngle - 360
      Else
         xAngle = xAngle - 6
         If xAngle < 0 Then xAngle = xAngle + 360
      End If
      Redo = True

   ' y rotates counterclockwise around y axis, Y rotates clockwise
   ElseIf fg_kbtest(21) = 1 Then
      If ShiftKey Then
         yAngle = yAngle + 6
         If yAngle >= 360 Then yAngle = yAngle - 360
      Else
         yAngle = yAngle - 6
         If yAngle < 0 Then yAngle = yAngle + 360
      End If
      Redo = True

   ' z rotates counterclockwise around z axis, Z rotates clockwise
   ElseIf fg_kbtest(44) = 1 Then
      If ShiftKey Then
         zAngle = zAngle + 6
         If zAngle >= 360 Then zAngle = zAngle - 360
      Else
        zAngle = zAngle - 6
        If zAngle < 0 Then zAngle = zAngle + 360
      End If
      Redo = True
   End If

   ' if rotation changed, modify the transformation matrix accordingly
   If Redo Then
      Call fg_rotate3d(Transform(1), xAngle * 10, yAngle * 10, zAngle * 10, 0)
      Redo = False
   End If
End Sub

'*****************************************************************************
'                                                                            *
'  DrawCube()                                                                *
'                                                                            *
'  Calls DrawFace() to project and draw each of the six cube faces.          *
'                                                                            *
'*****************************************************************************

Private Sub DrawCube()
   Call DrawFace(Face1, 84)
   Call DrawFace(Face2, 88)
   Call DrawFace(Face3, 92)
   Call DrawFace(Face4, 96)
   Call DrawFace(Face5, 100)
   Call DrawFace(Face6, 104)
End Sub

'*****************************************************************************
'                                                                            *
'  DrawFace()                                                                *
'                                                                            *
'  Called from DrawCube() to project and draw an individual cube face.       *
'                                                                            *
'*****************************************************************************

Private Sub DrawFace(Face() As Point3D, Color As Long)
   Dim ProjectedFace(4) As Point2D

   Call fg_project(Transform(1), Face(1).X, ProjectedFace(1).X, 4)
   Call SetColor(Color)
   Call fg_polyfill(ProjectedFace(1).X, WorkArray(1).X, -4)
End Sub

'*****************************************************************************
'                                                                            *
'  SetColor()                                                                *
'                                                                            *
'  Establish the current color for either a palette-based or direct color    *
'  virtual buffer.                                                           *
'                                                                            *
'*****************************************************************************

Private Sub SetColor(Color As Long)
   Dim R As Long, G As Long, B As Long

   If bpp > 8 Then
      Call fg_getrgb(Color, R, G, B)
      Call fg_setcolor(fg_maprgb(R, G, B))
   Else
      Call fg_setcolor(Color)
   End If
End Sub

'*****************************************************************************
'                                                                            *
'  InitGlobals()                                                             *
'                                                                            *
'  Initialize global variables and arrays. Called from Form_Load().          *
'                                                                            *
'*****************************************************************************

Private Sub InitGlobals()
   Redo = False
   xAngle = 0
   yAngle = 0
   zAngle = 0

   Face1(1).X = &HFFEC0000: Face1(1).Y = &H140000: Face1(1).Z = &H140000
   Face1(2).X = &H140000: Face1(2).Y = &H140000: Face1(2).Z = &H140000
   Face1(3).X = &H140000: Face1(3).Y = &HFFEC0000: Face1(3).Z = &H140000
   Face1(4).X = &HFFEC0000: Face1(4).Y = &HFFEC0000: Face1(4).Z = &H140000

   Face2(1).X = &H140000: Face2(1).Y = &H140000: Face2(1).Z = &H140000
   Face2(2).X = &H140000: Face2(2).Y = &H140000: Face2(2).Z = &HFFEC0000
   Face2(3).X = &H140000: Face2(3).Y = &HFFEC0000: Face2(3).Z = &HFFEC0000
   Face2(4).X = &H140000: Face2(4).Y = &HFFEC0000: Face2(4).Z = &H140000

   Face3(1).X = &HFFEC0000: Face3(1).Y = &HFFEC0000: Face3(1).Z = &HFFEC0000
   Face3(2).X = &H140000: Face3(2).Y = &HFFEC0000: Face3(2).Z = &HFFEC0000
   Face3(3).X = &H140000: Face3(3).Y = &H140000: Face3(3).Z = &HFFEC0000
   Face3(4).X = &HFFEC0000: Face3(4).Y = &H140000: Face3(4).Z = &HFFEC0000

   Face4(1).X = &HFFEC0000: Face4(1).Y = &H140000: Face4(1).Z = &HFFEC0000
   Face4(2).X = &HFFEC0000: Face4(2).Y = &H140000: Face4(2).Z = &H140000
   Face4(3).X = &HFFEC0000: Face4(3).Y = &HFFEC0000: Face4(3).Z = &H140000
   Face4(4).X = &HFFEC0000: Face4(4).Y = &HFFEC0000: Face4(4).Z = &HFFEC0000

   Face5(1).X = &HFFEC0000: Face5(1).Y = &H140000: Face5(1).Z = &HFFEC0000
   Face5(2).X = &H140000: Face5(2).Y = &H140000: Face5(2).Z = &HFFEC0000
   Face5(3).X = &H140000: Face5(3).Y = &H140000: Face5(3).Z = &H140000
   Face5(4).X = &HFFEC0000: Face5(4).Y = &H140000: Face5(4).Z = &H140000

   Face6(1).X = &HFFEC0000: Face6(1).Y = &HFFEC0000: Face6(1).Z = &H140000
   Face6(2).X = &H140000: Face6(2).Y = &HFFEC0000: Face6(2).Z = &H140000
   Face6(3).X = &H140000: Face6(3).Y = &HFFEC0000: Face6(3).Z = &HFFEC0000
   Face6(4).X = &HFFEC0000: Face6(4).Y = &HFFEC0000: Face6(4).Z = &HFFEC0000
End Sub
