!*****************************************************************************
!                                                                            *
!  CUBE2.F90                                                                 *
!                                                                            *
!  This program uses Fastgraph's 3D geometry system to draw a cube. Each     *
!  of the six cube faces is a different solid color. Uses the approach of    *
!  projecting the entire cube in one operation to take advantage of shared   *
!  vertices between faces.                                                   *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'cube2.fi'

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGcube2'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Rotating Cube'C,        &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   WIDTH/2,                 &  ! initial x size
   HEIGHT/2,                &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

! The message loop processes entries placed in the message queue.
! When no message is ready, call check_for_movement() to check if
! we want to perform movement.

do while (.TRUE.)
   if (PeekMessage(msg,NULL,0,0,PM_REMOVE)) then
      if (msg.message == WM_QUIT) then
         WinMain = msg%wParam
         return
      else
         iret = TranslateMessage(msg)
         iret = DispatchMessage(msg)
      end if
   else
      call check_for_movement
   end if
end do

end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'cube2.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal, hvb
integer*4 iret

integer*4 cxClient, cyClient, bpp
common    cxClient, cyClient, bpp

save hdc, hpal, hvb

WindowProc = 0

select case (message)

   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      bpp = fg_colors()
      if (bpp > 8) call fg_vbdepth(bpp)
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors
      call fg_setcolor(-1)
      call fg_fillpage

      ! define 3D viewport and initial transformation matrix
      call fg_view3d(0,WIDTH-1,0,HEIGHT-1,#00008000)
      call fg_rotate3d(transform,0,0,0,0)
      call fg_move3d(transform,0,0,#FF9C0000,0)

      redo = .FALSE.
      xAngle = 0
      yAngle = 0
      zAngle = 0
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  check_for_movement()                                                      *
!                                                                            *
!  The check_for_movement() function checks if any of the four arrow keys    *
!  are pressed, and if so, pans in that direction if possible. It is called  *
!  from the message loop in WinMain when no messages are waiting.            *
!                                                                            *
!*****************************************************************************

subroutine check_for_movement
use fgwin

include 'cube2.fi'

integer*4 cxClient, cyClient, bpp
common    cxClient, cyClient, bpp

logical*1 shift_key

! erase the previous frame from the virtual buffer
call fg_setcolor(-1)
call fg_fillpage

! project the cube onto the virtual buffer and draw it
call draw_cube

! display what we just drew
call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)

! check if either shift key is pressed
shift_key = (fg_kbtest(42) == 1) .or. (fg_kbtest(54) == 1)

! + and - move cube along the z axis (+ is toward viewer, - is
! away from viewer), restricting movement beyond z=-40
if (fg_kbtest(74) == 1) then
   call fg_move3d(transform,0,0,#FFFD0000,1)
else if (fg_kbtest(78) == 1) then
   if (transform(12) < #FFD80000) then
      call fg_move3d(transform,0,0,#00030000,1)
   end if

! left and right arrow keys move cube along x axis
else if (fg_kbtest(75) == 1) then
   call fg_move3d(transform,#FFFD0000,0,0,1)
else if (fg_kbtest(77) == 1) then
   call fg_move3d(transform,#00030000,0,0,1)

! up and down arrow keys move cube along y axis
else if (fg_kbtest(72) == 1) then
   call fg_move3d(transform,0,#00030000,0,1)
else if (fg_kbtest(80) == 1) then
   call fg_move3d(transform,0,#FFFD0000,0,1)

! x rotates counterclockwise around x axis, X rotates clockwise
else if (fg_kbtest(45) == 1) then
   if (shift_key) then
      xAngle = xAngle + 6
      if (xAngle >= 360) xAngle = xAngle - 360
   else
      xAngle = xAngle - 6
      if (xAngle < 0) xAngle = xAngle + 360
   end if
   redo = .TRUE.

! y rotates counterclockwise around y axis, Y rotates clockwise
else if (fg_kbtest(21) == 1) then
   if (shift_key) then
      yAngle = yAngle + 6
      if (yAngle >= 360) yAngle = yAngle - 360
   else
      yAngle = yAngle - 6
      if (yAngle < 0) yAngle = yAngle + 360
   end if
   redo = .TRUE.

! z rotates counterclockwise around z axis, Z rotates clockwise
else if (fg_kbtest(44) == 1) then
   if (shift_key) then
      zAngle = zAngle + 6
      if (zAngle >= 360) zAngle = zAngle - 360
   else
      zAngle = zAngle - 6
      if (zAngle < 0) zAngle = zAngle + 360
   end if
   redo = .TRUE.
end if

! if rotation changed, modify the transformation matrix accordingly
if (redo) then
   call fg_rotate3d(transform,xAngle*10,yAngle*10,zAngle*10,0)
   redo = .FALSE.
end if

return
end

!*****************************************************************************
!                                                                            *
!  draw_cube()                                                               *
!                                                                            *
!  Draws each of the six cube faces. For each face, we obtain the projected  *
!  coordinates from the projected_cube array and store them in the vertices  *
!  array so we can draw the cube face with fg_polyfill().                    *
!                                                                            *
!*****************************************************************************

subroutine draw_cube
use fgwin

include 'cube2.fi'

integer*4 cxClient, cyClient, bpp
common    cxClient, cyClient, bpp

integer*4 i, j, k
integer*4 r, g, b
integer*4 vertices(8)
integer*4 projected_cube(16)
integer*4 colors(6)
data colors /84,88,92,96,100,104/

call fg_project(transform,cube,projected_cube,8)
k = 1
do i = 1,6
   do j = 1,7,2
      vertices(j)   = projected_cube(faces(k))    ! x coordinate
      vertices(j+1) = projected_cube(faces(k)+1)  ! y coordinate
      k = k + 1
   end do
   if (bpp > 8) then
      call fg_getrgb(colors(i),r,g,b)
      call fg_setcolor(fg_maprgb(r,g,b))
   else
      call fg_setcolor(colors(i))
   end if
   call fg_polyfill(vertices,work_array,-4)
end do

return
end

!*****************************************************************************

block data

include 'cube2.fi'

! vertices of a 40x40x40 cube, defined in 3D coordinates
data cube / &
   #FFEC0000, #00140000, #00140000, &
   #00140000, #00140000, #00140000, &
   #FFEC0000, #FFEC0000, #00140000, &
   #00140000, #FFEC0000, #00140000, &
   #FFEC0000, #00140000, #FFEC0000, &
   #00140000, #00140000, #FFEC0000, &
   #FFEC0000, #FFEC0000, #FFEC0000, &
   #00140000, #FFEC0000, #FFEC0000/

! indices into the cube array that define each face of the cube
data face1 / 1, 3, 7, 5/
data face2 / 3,11,15, 7/
data face3 /13,15,11, 9/
data face4 / 9, 1, 5,13/
data face5 / 9,11, 3, 1/
data face6 / 5, 7,15,13/

end
