/****************************************************************************\
*                                                                            *
*  CUBE2.CPP                                                                 *
*  CUBE2U.CPP                                                                *
*                                                                            *
*  This program uses Fastgraph's 3D geometry system to draw a cube. Each     *
*  of the six cube faces is a different solid color. Uses the approach of    *
*  projecting the entire cube in one operation to take advantage of shared   *
*  vertices between faces.                                                   *
*                                                                            *
\****************************************************************************/

#include <vcl\vcl.h>
#pragma hdrstop

#include "Cube2U.h"
#define INT2FIX(n) ((long)(n) << 16)

typedef long FIXEDPOINT;

typedef struct point2d
{
   int x;
   int y;
} POINT2D;

typedef struct point3d
{
   FIXEDPOINT x;
   FIXEDPOINT y;
   FIXEDPOINT z;
} POINT3D;

// 3D transformation matrix
FIXEDPOINT transform[12];

// vertices of a 40x40x40 cube, defined in 3D coordinates
POINT3D cube[] = {
   {INT2FIX(-20), INT2FIX( 20), INT2FIX( 20)},
   {INT2FIX( 20), INT2FIX( 20), INT2FIX( 20)},
   {INT2FIX(-20), INT2FIX(-20), INT2FIX( 20)},
   {INT2FIX( 20), INT2FIX(-20), INT2FIX( 20)},
   {INT2FIX(-20), INT2FIX( 20), INT2FIX(-20)},
   {INT2FIX( 20), INT2FIX( 20), INT2FIX(-20)},
   {INT2FIX(-20), INT2FIX(-20), INT2FIX(-20)},
   {INT2FIX( 20), INT2FIX(-20), INT2FIX(-20)}
};

// indices into the cube array that define each face of the cube
int face1[] = {0,1,3,2};
int face2[] = {1,5,7,3};
int face3[] = {6,7,5,4};
int face4[] = {4,0,2,6};
int face5[] = {4,5,1,0};
int face6[] = {2,3,7,6};

// for convenience, an array of pointers to each of the six faces
int *faces[] = {face1,face2,face3,face4,face5,face6};

// work array required by fg_polyfill()
POINT2D work_array[1000];
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner),
    WIDTH(640),
    HEIGHT(480)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OnIdle(TObject *Sender, bool &Done)
{
   check_for_movement();
   Done = False;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormActivate(TObject *Sender)
{
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
   hdc = GetDC(Form1->Handle);
   fg_setdc(hdc);
   hpal = fg_defpal();
   fg_realize(hpal);

   fg_vbinit();
   bpp = fg_colors();
   if (bpp > 8) fg_vbdepth(bpp);
   hvb = fg_vballoc(WIDTH,HEIGHT);
   fg_vbopen(hvb);
   fg_vbcolors();
   fg_setcolor(-1);
   fg_fillpage();

   // define 3D viewport and initial transformation matrix
   fg_view3d(0,WIDTH-1,0,HEIGHT-1,0x00008000);
   fg_rotate3d(transform,0,0,0,0);
   fg_move3d(transform,0,0,INT2FIX(-100),0);

   Application->OnActivate = OnActivate;
   Application->OnIdle = OnIdle;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormPaint(TObject *Sender)
{
   fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormResize(TObject *Sender)
{
   cxClient = ClientWidth;
   cyClient = ClientHeight;
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   fg_vbclose();
   fg_vbfree(hvb);
   fg_vbfin();
   DeleteObject(hpal);
   ReleaseDC(Form1->Handle,hdc);
}

/****************************************************************************\
*                                                                            *
*  check_for_movement()                                                      *
*                                                                            *
*  The check_for_movement() function checks if any of the four arrow keys    *
*  are pressed, and if so, pans in that direction if possible. It is called  *
*  from the message loop in WinMain when no messages are waiting.            *
*                                                                            *
\****************************************************************************/

bool redo = False;
bool shift_key = False;
int  xAngle = 0, yAngle = 0, zAngle = 0;

void  TForm1::check_for_movement()
{
   // erase the previous frame from the virtual buffer
   fg_setcolor(-1);
   fg_fillpage();

   // project the cube onto the virtual buffer and draw it
   draw_cube();

   // display what we just drew
   fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1);

   // check if either shift key is pressed
   shift_key = fg_kbtest(42) | fg_kbtest(54);

   // + and - move cube along the z axis (+ is toward viewer, - is
   // away from viewer), restricting movement beyond z=-40
   if (fg_kbtest(74))
      fg_move3d(transform,0,0,INT2FIX(-3),1);
   else if (fg_kbtest(78))
   {
      if (transform[11] < INT2FIX(-40))
         fg_move3d(transform,0,0,INT2FIX(3),1);
   }

   // left and right arrow keys move cube along x axis
   else if (fg_kbtest(75))
      fg_move3d(transform,INT2FIX(-3),0,0,1);
   else if (fg_kbtest(77))
      fg_move3d(transform,INT2FIX(3),0,0,1);

   // up and down arrow keys move cube along y axis
   else if (fg_kbtest(72))
      fg_move3d(transform,0,INT2FIX(3),0,1);
   else if (fg_kbtest(80))
      fg_move3d(transform,0,INT2FIX(-3),0,1);

   // x rotates counterclockwise around x axis, X rotates clockwise
   else if (fg_kbtest(45))
   {
      if (shift_key)
      {
         xAngle += 6;
         if (xAngle >= 360) xAngle -= 360;
      }
      else
      {
         xAngle -= 6;
         if (xAngle < 0) xAngle += 360;
      }
      redo = True;
   }

   // y rotates counterclockwise around y axis, Y rotates clockwise
   else if (fg_kbtest(21))
   {
      if (shift_key)
      {
         yAngle += 6;
         if (yAngle >= 360) yAngle -= 360;
      }
      else
      {
         yAngle -= 6;
         if (yAngle < 0) yAngle += 360;
      }
      redo = True;
   }

   // z rotates counterclockwise around z axis, Z rotates clockwise
   else if (fg_kbtest(44))
   {
      if (shift_key)
      {
         zAngle += 6;
         if (zAngle >= 360) zAngle -= 360;
      }
      else
      {
         zAngle -= 6;
         if (zAngle < 0) zAngle += 360;
      }
      redo = True;
   }

   // if rotation changed, modify the transformation matrix accordingly
   if (redo)
   {
      fg_rotate3d(transform,xAngle*10,yAngle*10,zAngle*10,0);
      redo = False;
   }
}
/****************************************************************************\
*                                                                            *
*  draw_cube()                                                               *
*                                                                            *
*  Draws each of the six cube faces. For each face, we obtain the projected  *
*  coordinates from the projected_cube array and store them in the vertices  *
*  array so we can draw the cube face with fg_polyfill().                    *
*                                                                            *
\****************************************************************************/

void TForm1::draw_cube()
{
   register int i, j;
   int r, g, b;
   POINT2D vertices[4];
   POINT2D projected_cube[8];
   static int colors[] = {80,84,88,92,96,100};

   fg_project(transform,(FIXEDPOINT *)cube,(int *)projected_cube,8);
   for (i = 0; i < 6; i++)
   {
      for (j = 0; j < 4; j++)
      {
         vertices[j].x = projected_cube[faces[i][j]].x;
         vertices[j].y = projected_cube[faces[i][j]].y;
      }
      if (bpp > 8)
      {
         fg_getrgb(colors[i],&r,&g,&b);
         fg_setcolor(fg_maprgb(r,g,b));
      }
      else
      {
         fg_setcolor(colors[i]);
      }
      fg_polyfill((int *)vertices,work_array,-4);
   }
}
