/****************************************************************************\
*                                                                            *
*  FRAMEDD.CPP                                                               *
*  FRAMEDDU.CPP                                                              *
*                                                                            *
*  This program shows how to set up a full screen DirectDraw application     *
*  for either blitting or flipping. The selection of blitting or flipping is *
*  controlled by the BLIT and FLIP symbols defined below.                    *
*                                                                            *
\****************************************************************************/

#include <vcl\vcl.h>
#pragma hdrstop

#include "FrameDDU.h"

// define either BLIT or FLIP, but not both, for blitting or flipping
#define BLIT
//#define FLIP

//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner),
    WIDTH(640),
    HEIGHT(480)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OnActivate(TObject *Sender)
{
   AppIsActive = True;
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OnDeactivate(TObject *Sender)
{
   AppIsActive = False;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OnIdle(TObject *Sender, bool &Done)
{
   if (AppIsActive) animate();
   Done = False;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormActivate(TObject *Sender)
{
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
#ifdef BLIT
   fg_ddsetup(WIDTH,HEIGHT,8,0);
#else
   fg_ddsetup(WIDTH,HEIGHT,8,1);
#endif
   hdc = GetDC(Form1->Handle);
   fg_setdc(hdc);
   hpal = fg_defpal();
   fg_realize(hpal);
   Form1->Visible = True;

   fg_vbinit();
   // if blitting, create a virtual buffer the same size as the screen
   // resolution; if flipping, use the primary surface's back buffer
#ifdef BLIT
   hvb = fg_vballoc(WIDTH,HEIGHT);
#else
   hvb = 0;
#endif
   fg_vbopen(hvb);
   fg_vbcolors();

   fg_setcolor(19);
   fg_fillpage();
   fg_mouseini();
   fg_mousevis(0);

   AppIsActive = True;
   Application->OnActivate = OnActivate;
   Application->OnDeactivate = OnDeactivate;
   Application->OnIdle = OnIdle;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormKeyDown(TObject *Sender, WORD &Key,
	TShiftState Shift)
{
  if (Key == VK_ESCAPE || Key == VK_F12) Application->Terminate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   fg_mousevis(1);
   fg_vbclose();
#ifdef BLIT
   fg_vbfree(hvb);
#endif
   fg_vbfin();
   DeleteObject(hpal);
   ReleaseDC(Form1->Handle,hdc);
}

/****************************************************************************\
*                                                                            *
*  animate()                                                                 *
*                                                                            *
*  Construct the next frame of animation and display it with either blitting *
*  or flipping, as directed by the BLIT and FLIP symbols above.              *
*                                                                            *
\****************************************************************************/

void TForm1::animate()
{
   // fill drawing surface with the next color
   fg_setcolor((fg_getcolor() + 1) & 0xFF);
   fg_fillpage();

   // blit or flip the surface to the screen
#ifdef BLIT
   fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1);
#else
   fg_ddflip();
#endif
}
//---------------------------------------------------------------------------
