{*****************************************************************************
*                                                                            *
*  IMAGE.DPR                                                                 *
*  IMAGEU.PAS                                                                *
*                                                                            *
*  This program demonstrates the Fastgraph for Windows image file display    *
*  and creation functions.                                                   *
*                                                                            *
*****************************************************************************}

unit ImageU;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, CommDlg, Menus, FGWin;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    BMP1: TMenuItem;
     Open1: TMenuItem;
     Make1: TMenuItem;
     Info1: TMenuItem;
    PCX1: TMenuItem;
     Open2: TMenuItem;
     Make2: TMenuItem;
     Info2: TMenuItem;
    FLIFLC1: TMenuItem;
     Open3: TMenuItem;
     Play3: TMenuItem;
     Frame3: TMenuItem;
     Reset3: TMenuItem;
     Info3: TMenuItem;
    JPEG1: TMenuItem;
     Open4: TMenuItem;
     Info4: TMenuItem;
    Quit1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure AppOnActivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BMPopen(Sender: TObject);
    procedure BMPmake(Sender: TObject);
    procedure BMPinfo(Sender: TObject);
    procedure PCXopen(Sender: TObject);
    procedure PCXmake(Sender: TObject);
    procedure PCXinfo(Sender: TObject);
    procedure FLICopen(Sender: TObject);
    procedure FLICplay(Sender: TObject);
    procedure FLICframe(Sender: TObject);
    procedure FLICreset(Sender: TObject);
    procedure FLICinfo(Sender: TObject);
    procedure JPEGopen(Sender: TObject);
    procedure JPEGinfo(Sender: TObject);
    procedure Quit(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
var
  cxClient, cyClient : integer;
  cxBuffer, cyBuffer : integer;
  colors : integer;
  dc     : hDC;
  hpal   : hPalette;
  hvb    : integer;

  file_header  : array [0..127] of byte;
  file_palette : array [1..768] of byte;
  file_name    : string;
  open_file    : string;
  mb_text      : string;

  context : array [1..16] of byte;
  frames  : integer;

{*****************************************************************************
*                                                                            *
*  switch_buffers                                                            *
*                                                                            *
*  Close the and release the active virtual buffer, then create and open a   *
*  new virtual buffer to hold the new image file.                            *
*                                                                            *
*****************************************************************************}

procedure switch_buffers;
begin
  fg_vbclose;
  fg_vbfree(hvb);
  if (colors = 0) then
    fg_vbdepth(24)
  else
    fg_vbdepth(8);
  hvb := fg_vballoc(cxBuffer,cyBuffer);
  fg_vbopen(hvb);
  fg_vbcolors;
end;

{****************************************************************************}

procedure TForm1.AppOnActivate(Sender: TObject);
begin
  fg_realize(hpal);
  Invalidate;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  fg_realize(hpal);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  dc := GetDC(Form1.Handle);
  fg_setdc(dc);
  hpal := fg_defpal;
  fg_realize(hpal);

  fg_vbinit;
  hvb := fg_vballoc(4,4);
  fg_vbopen(hvb);
  fg_vbcolors;

  fg_setcolor(25);
  fg_fillpage;

{$IFDEF VER80}
  Form1.Open4.Enabled := False;
{$ENDIF}
  Application.OnActivate := AppOnActivate;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  fg_vbscale(0,fg_getmaxx,0,fg_getmaxy,0,cxClient-1,0,cyClient-1);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  cxClient := ClientWidth;
  cyClient := ClientHeight;
  Invalidate;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fg_vbclose;
  fg_vbfree(hvb);
  fg_vbfin;
  DeleteObject(hpal);
  ReleaseDC(Form1.Handle,dc);
end;

{*****************************************************************************
*                                                                            *
*  Event handlers for the items on the BMP menu.                             *
*                                                                            *
*****************************************************************************}

procedure TForm1.BMPopen(Sender: TObject);
begin
  OpenDialog1.DefaultExt := 'bmp';
  OpenDialog1.FileName := '';
  OpenDialog1.Filter := 'BMP files (*.BMP)|*.BMP';
  OpenDialog1.Options := [ofReadOnly];
  if (OpenDialog1.Execute = False) then Exit;
  file_name := OpenDialog1.FileName;
  if (fg_bmphead(file_name,file_header) < 0) then
  begin
    mb_text := file_name + chr(13) + 'is not a BMP file.';
    MessageDlg(mb_text,mtError,[mbOK],0);
    Exit;
  end;
  file_name := OpenDialog1.FileName;
  colors := fg_bmppal(file_name,file_palette);
{$IFDEF VER80}
  if (colors = 0) then
  begin
    mb_text := file_name + chr(13) + 'is a 24-bit BMP file.';
    MessageDlg(mb_text,mtError,[mbOK],0);
    Exit;
  end;
{$ENDIF}
  Form1.Cursor := crHourglass;
  fg_bmpsize(file_header,cxBuffer,cyBuffer);
  switch_buffers;
  fg_showbmp(file_name,0);
  fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
  Form1.Cursor := crDefault;
  open_file := file_name;

  Form1.Make1.Enabled  := True;
  Form1.Info1.Enabled  := True;
  Form1.Make2.Enabled  := True;
  Form1.Info2.Enabled  := False;
  Form1.Play3.Enabled  := False;
  Form1.Frame3.Enabled := False;
  Form1.Reset3.Enabled := False;
  Form1.Info3.Enabled  := False;
  Form1.Info4.Enabled  := False;
end;

procedure TForm1.BMPmake(Sender: TObject);
var
  color_depth : integer;
begin
  SaveDialog1.DefaultExt := 'bmp';
  SaveDialog1.FileName := ChangeFileExt(file_name,'');
  SaveDialog1.Filter := 'BMP files (*.BMP)|*.BMP';
  SaveDialog1.Options := [ofHideReadOnly,ofOverwritePrompt,ofPathMustExist];
  if (SaveDialog1.Execute = False) then Exit;
  file_name := SaveDialog1.FileName;
  if (colors = 0) then
    color_depth := 24
  else if (colors = 256) then
    color_depth := 8
  else if (colors = 16) then
    color_depth := 4
  else
    color_depth := 1;
  Form1.Cursor := crHourglass;
  fg_makebmp(0,cxBuffer-1,0,cyBuffer-1,color_depth,file_name);
  Form1.Cursor := crDefault;
end;

procedure TForm1.BMPinfo(Sender: TObject);
var
  s : string;
begin
  Str(cxBuffer,s);
  mb_text := open_file + chr(13) + s + 'x';
  Str(cyBuffer,s);
  mb_text := mb_text + s + ' pixels' + chr(13);
  if (colors > 0) then
  begin
    Str(colors,s);
    mb_text := mb_text + s + ' colors';
  end
  else
    mb_text := mb_text + '24-bit RGB';
  MessageDlg(mb_text,mtInformation,[mbOK],0);
end;

{*****************************************************************************
*                                                                            *
*  Event handlers for the items on the PCX menu.                             *
*                                                                            *
*****************************************************************************}

procedure TForm1.PCXopen(Sender: TObject);
begin
  OpenDialog1.DefaultExt := 'pcx';
  OpenDialog1.FileName := '';
  OpenDialog1.Filter := 'PCX files (*.PCX)|*.PCX';
  OpenDialog1.Options := [ofReadOnly];
  if (OpenDialog1.Execute = False) then Exit;
  file_name := OpenDialog1.FileName;
  if (fg_pcxhead(file_name,file_header) < 0) then
  begin
    mb_text := file_name + chr(13) + 'is not a PCX file.';
    MessageDlg(mb_text,mtError,[mbOK],0);
    Exit;
  end;
  colors := fg_pcxpal(file_name,file_palette);
{$IFDEF VER80}
  if (colors = 0) then
  begin
    mb_text := file_name + chr(13) + 'is a 24-bit PCX file.';
    MessageDlg(mb_text,mtError,[mbOK],0);
    Exit;
  end;
{$ENDIF}
  Form1.Cursor := crHourglass;
  fg_pcxsize(file_header,cxBuffer,cyBuffer);
  switch_buffers;
  fg_move(0,0);
  fg_showpcx(file_name,2);
  fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
  Form1.Cursor := crDefault;
  open_file := file_name;

  Form1.Make1.Enabled  := True;
  Form1.Info1.Enabled  := False;
  Form1.Make2.Enabled  := True;
  Form1.Info2.Enabled  := True;
  Form1.Play3.Enabled  := False;
  Form1.Frame3.Enabled := False;
  Form1.Reset3.Enabled := False;
  Form1.Info3.Enabled  := False;
  Form1.Info4.Enabled  := False;
end;

procedure TForm1.PCXmake(Sender: TObject);
begin
  SaveDialog1.DefaultExt := 'pcx';
  SaveDialog1.FileName := ChangeFileExt(file_name,'');
  SaveDialog1.Filter := 'PCX files (*.PCX)|*.PCX';
  SaveDialog1.Options := [ofHideReadOnly,ofOverwritePrompt,ofPathMustExist];
  if (SaveDialog1.Execute = False) then Exit;
  file_name := SaveDialog1.FileName;
  Form1.Cursor := crHourglass;
  fg_makepcx(0,cxBuffer-1,0,cyBuffer-1,file_name);
  Form1.Cursor := crDefault;
end;

procedure TForm1.PCXinfo(Sender: TObject);
var
  s : string;
begin
  Str(cxBuffer,s);
  mb_text := open_file + chr(13) + s + 'x';
  Str(cyBuffer,s);
  mb_text := mb_text + s + ' pixels' + chr(13);
  if (colors > 0) then
  begin
    Str(colors,s);
    mb_text := mb_text + s + ' colors';
  end
  else
    mb_text := mb_text + '24-bit RGB';
  MessageDlg(mb_text,mtInformation,[mbOK],0);
end;

{*****************************************************************************
*                                                                            *
*  Event handlers for the items on the FLI/FLC menu.                         *
*                                                                            *
*****************************************************************************}

procedure TForm1.FLICopen(Sender: TObject);
begin
  OpenDialog1.DefaultExt := 'fli';
  OpenDialog1.FileName := '';
  OpenDialog1.Filter := 'flic files (*.FLI,*.FLC)|*.FLI;*.FLC';
  OpenDialog1.Options := [ofReadOnly];
  if (OpenDialog1.Execute = False) then Exit;
  file_name := OpenDialog1.FileName;
  if (fg_flichead(file_name,file_header) < 0) then
  begin
    mb_text := file_name + chr(13) + 'is not an FLI or FLC file.';
    MessageDlg(mb_text,mtError,[mbOK],0);
    Exit;
  end;
  colors := 256;
  fg_flicsize(file_header,cxBuffer,cyBuffer);
  switch_buffers;
  fg_flicopen(file_name,context);
  fg_flicplay(context,1,0);
  fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
  Move(file_header[6],frames,2);
  open_file := file_name;

  Form1.Make1.Enabled  := True;
  Form1.Info1.Enabled  := False;
  Form1.Make2.Enabled  := True;
  Form1.Info2.Enabled  := False;
  Form1.Play3.Enabled  := True;
  Form1.Frame3.Enabled := True;
  Form1.Reset3.Enabled := True;
  Form1.Info3.Enabled  := True;
  Form1.Info4.Enabled  := False;
end;

procedure TForm1.FLICplay(Sender: TObject);
begin
  Form1.Cursor := crHourglass;
  fg_showflic(file_name,0,1);
  Form1.Cursor := crDefault;
  fg_flicskip(context,-1);
end;

procedure TForm1.FLICframe(Sender: TObject);
begin
  if (fg_flicplay(context,1,0) = 0) then
  begin
    fg_flicskip(context,-1);
    fg_flicplay(context,1,0);
  end;
  fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
end;

procedure TForm1.FLICreset(Sender: TObject);
begin
  fg_flicskip(context,-1);
  fg_flicplay(context,1,0);
  fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
end;

procedure TForm1.FLICinfo(Sender: TObject);
var
  s : string;
begin
  Str(cxBuffer,s);
  mb_text := open_file + chr(13) + s + 'x';
  Str(cyBuffer,s);
  mb_text := mb_text + s + ' pixels' + chr(13);
  Str(frames,s);
  mb_text := mb_text + s + ' frames';
  MessageDlg(mb_text,mtInformation,[mbOK],0);
end;

{*****************************************************************************
*                                                                            *
*  Event handlers for the items on the JPEG menu.                            *
*                                                                            *
*****************************************************************************}

procedure TForm1.JPEGopen(Sender: TObject);
var
  buffer : pointer;
  buffsize : integer;
begin
{$IFNDEF VER80}
  OpenDialog1.DefaultExt := 'jpg';
  OpenDialog1.FileName := '';
  OpenDialog1.Filter := 'JPEG files (*.JPG)|*.JPG';
  OpenDialog1.Options := [ofReadOnly];
  if (OpenDialog1.Execute = False) then Exit;
  file_name := OpenDialog1.FileName;
  if (fg_jpeghead(file_name,file_header) < 0) then
  begin
    mb_text := file_name + chr(13) + 'is not a JPEG file.';
    MessageDlg(mb_text,mtError,[mbOK],0);
    Exit;
  end;
  colors := 0;
  Form1.Cursor := crHourglass;
  buffsize := fg_jpegmem(file_header);
  GetMem(buffer,buffsize);
  fg_jpegbuf(buffer,buffsize);
  fg_jpegsize(file_header,cxBuffer,cyBuffer);
  switch_buffers;
  fg_move(0,0);
  fg_showjpeg(file_name,0);
  FreeMem(buffer,buffsize);
  fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1);
  Form1.Cursor := crDefault;
  open_file := file_name;

  Form1.Make1.Enabled  := True;
  Form1.Info1.Enabled  := False;
  Form1.Make2.Enabled  := True;
  Form1.Info2.Enabled  := False;
  Form1.Play3.Enabled  := False;
  Form1.Frame3.Enabled := False;
  Form1.Reset3.Enabled := False;
  Form1.Info3.Enabled  := False;
  Form1.Info4.Enabled  := True;
{$ENDIF}
end;

procedure TForm1.JPEGinfo(Sender: TObject);
var
  s : string;
begin
{$IFNDEF VER80}
  Str(cxBuffer,s);
  mb_text := open_file + chr(13) + s + 'x';
  Str(cyBuffer,s);
  mb_text := mb_text + s + ' pixels' + chr(13) + '24-bit RGB';
  MessageDlg(mb_text,mtInformation,[mbOK],0);
{$ENDIF}
end;

{****************************************************************************}

procedure TForm1.Quit(Sender: TObject);
begin
  Application.Terminate;
end;

end.
