/****************************************************************************\
*                                                                            *
*  KBDEMO.CPP                                                                *
*  KBDEMOU.CPP                                                               *
*                                                                            *
*  This program shows how to pan the contents of a virtual buffer through    *
*  a smaller window using the low-level keyboard handler.                    *
*                                                                            *
\****************************************************************************/

#include <vcl\vcl.h>
#pragma hdrstop

#include "KBDemoU.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner),
    WIDTH(640),
    HEIGHT(480)
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OnIdle(TObject *Sender, bool &Done)
{
   check_for_panning();
   Done = False;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormActivate(TObject *Sender)
{
   fg_realize(hpal);
   Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
   hdc = GetDC(Form1->Handle);
   fg_setdc(hdc);
   hpal = fg_defpal();
   fg_realize(hpal);

   fg_vbinit();
   hvb = fg_vballoc(WIDTH,HEIGHT);
   fg_vbopen(hvb);
   fg_vbcolors();

   fg_showbmp("PORCH.BMP",0);
   x = y = 0;
   CanGoRight = CanGoDown = True;
   CanGoLeft = CanGoUp = False;

   Application->OnActivate = OnActivate;
   Application->OnIdle = OnIdle;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormPaint(TObject *Sender)
{
   fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormResize(TObject *Sender)
{
   cxClient = ClientWidth;
   cyClient = ClientHeight;
   Invalidate();

   if (cxClient < WIDTH)
   {
      xlimit = WIDTH - cxClient;
      if (x > 0)      CanGoLeft = True;
      if (x < xlimit) CanGoRight = True;
   }
   else
   {
      xlimit = 0;
      CanGoLeft = CanGoRight = False;
   }
   if (cyClient < HEIGHT)
   {
      ylimit = HEIGHT - cyClient;
      if (y > 0)      CanGoUp = True;
      if (y < ylimit) CanGoDown = True;
   }
   else
   {
      ylimit = 0;
      CanGoUp = CanGoDown = False;
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
   fg_vbclose();
   fg_vbfree(hvb);
   fg_vbfin();
   DeleteObject(hpal);
   ReleaseDC(Form1->Handle,hdc);
}

/****************************************************************************\
*                                                                            *
*  The check_for_panning() function checks if any of the four arrow keys are *
*  pressed, and if so, pans in that direction if possible. It is called from *
*  the OnIdle event handler.                                                 *
*                                                                            *
\****************************************************************************/

#define KB_ESCAPE 1
#define KB_LEFT  75
#define KB_RIGHT 77
#define KB_UP    72
#define KB_DOWN  80

void TForm1::check_for_panning()
{
   if (fg_kbtest(KB_LEFT) && CanGoLeft)
   {
      if (x == xlimit) CanGoRight = True;
      x--;
      fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1);
      if (x == 0) CanGoLeft = False;
   }
   else if (fg_kbtest(KB_RIGHT) && CanGoRight)
   {
      if (x == 0) CanGoLeft = True;
      x++;
      fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1);
      if (x == xlimit) CanGoRight = False;
   }
   else if (fg_kbtest(KB_UP) && CanGoUp)
   {
      if (y == ylimit) CanGoDown = True;
      y--;
      fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1);
      if (y == 0) CanGoUp = False;
   }
   else if (fg_kbtest(KB_DOWN) && CanGoDown)
   {
      if (y == 0) CanGoUp = True;
      y++;
      fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1);
      if (y == ylimit) CanGoDown = False;
   }
   else if (fg_kbtest(KB_ESCAPE))
   {
      x = y = 0;
      fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1);
      if (xlimit > 0) CanGoRight = True;
      if (ylimit > 0) CanGoDown = True;
      CanGoLeft = CanGoUp = False;
   }
}
