!*****************************************************************************
!                                                                            *
!  PANNER.F90                                                                *
!                                                                            *
!  This program shows how to pan the contents of a virtual buffer through    *
!  a smaller window.                                                         *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'panner.fi'

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGpanner'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = LOC(AppName)
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Panning Demo'C,         & ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   WIDTH/2,                 &  ! initial x size
   HEIGHT/2,                &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'panner.fi'

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hmenu, hpal, hvb
integer*4 iret
integer*4 x, y
integer*4 xlimit, ylimit

save cxClient, cyClient, hdc, hmenu, hpal, hvb
save x, y, xlimit, ylimit

WindowProc = 0

select case (message)

   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      iret = fg_showbmp('PORCH.BMP'C,0)
      x = 0
      y = 0

      hmenu = GetMenu(window)
      return

   case (WM_COMMAND)
      select case (wParam)
         case (IDM_LEFT)
            if (x == xlimit) then
               iret = EnableMenuItem(hmenu,IDM_RIGHT,MF_ENABLED)
               iret = DrawMenuBar(window)
            end if
            x = x - 1
            call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
            if (x == 0) then
               iret = EnableMenuItem(hmenu,IDM_LEFT,MF_GRAYED)
               iret = DrawMenuBar(window)
            end if
            return

         case (IDM_RIGHT)
            if (x == 0) then
               iret = EnableMenuItem(hmenu,IDM_LEFT,MF_ENABLED)
               iret = DrawMenuBar(window)
            end if
            x = x + 1
            call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
            if (x == xlimit) then
               iret = EnableMenuItem(hmenu,IDM_RIGHT,MF_GRAYED)
               iret = DrawMenuBar(window)
            end if
            return

         case (IDM_UP)
            if (y == ylimit) then
               iret = EnableMenuItem(hmenu,IDM_DOWN,MF_ENABLED)
               iret = DrawMenuBar(window)
            end if
            y = y - 1
            call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
            if (y == 0) then
               iret = EnableMenuItem(hmenu,IDM_UP,MF_GRAYED)
               iret = DrawMenuBar(window)
            end if
            return

         case (IDM_DOWN)
            if (y == 0) then
               iret = EnableMenuItem(hmenu,IDM_UP,MF_ENABLED)
               iret = DrawMenuBar(window)
            end if
            y = y + 1
            call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
            if (y == ylimit) then
               iret = EnableMenuItem(hmenu,IDM_DOWN,MF_GRAYED)
               iret = DrawMenuBar(window)
            end if
            return

         case (IDM_RESET)
            x = 0
            y = 0
            call fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,HEIGHT-1)
            if (xlimit > 0) iret = EnableMenuItem(hmenu,IDM_RIGHT,MF_ENABLED)
            if (ylimit > 0) iret = EnableMenuItem(hmenu,IDM_DOWN,MF_ENABLED)
            iret = EnableMenuItem(hmenu,IDM_LEFT,MF_GRAYED)
            iret = EnableMenuItem(hmenu,IDM_UP,MF_GRAYED)
            iret = DrawMenuBar(window)
            return

         case (IDM_QUIT)
            iret = DestroyWindow(window)
            return
      end select

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbpaste(x,x+(WIDTH-1),y,y+(HEIGHT-1),0,HEIGHT-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      if (cxClient < WIDTH) then
         xlimit = WIDTH - cxClient
         if (x > 0)      iret = EnableMenuItem(hmenu,IDM_LEFT,MF_ENABLED)
         if (x < xlimit) iret = EnableMenuItem(hmenu,IDM_RIGHT,MF_ENABLED)
      else
         xlimit = 0
         iret = EnableMenuItem(hmenu,IDM_LEFT,MF_GRAYED)
         iret = EnableMenuItem(hmenu,IDM_RIGHT,MF_GRAYED)
      end if
      if (cyClient < HEIGHT) then
         ylimit = HEIGHT - cyClient
         if (y > 0)      iret = EnableMenuItem(hmenu,IDM_UP,MF_ENABLED)
         if (y < ylimit) iret = EnableMenuItem(hmenu,IDM_DOWN,MF_ENABLED)
      else
         ylimit = 0
         iret = EnableMenuItem(hmenu,IDM_UP,MF_GRAYED)
         iret = EnableMenuItem(hmenu,IDM_DOWN,MF_GRAYED)
      end if
      iret = DrawMenuBar(window)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end
